@@ -11,7 +11,7 @@
 #define ds_deque_increment_tail(_d) (_d)->tail = ((_d)->tail + 1) & ((_d)->capacity - 1)
 #define ds_deque_decrement_tail(_d) (_d)->tail = ((_d)->tail - 1) & ((_d)->capacity - 1)
 
-static inline void _memmove(
+static inline void ds_deque_memmove(
     ds_deque_t *deque,
     zend_long   dst,
     zend_long   src,
@@ -20,7 +20,10 @@ static inline void _memmove(
     memmove(&deque->buffer[dst], &deque->buffer[src], length * sizeof(zval));
 }
 
-static zend_long ds_deque_required_capacity(zend_long n)
+/**
+ * Translates a given number to an acceptable deque capacity >= n.
+ */
+static zend_long ds_deque_get_capacity_for_size(zend_long n)
 {
     // We have to allocate an extra element for the tail, so a requested power
     // of two would need to round to the next.
@@ -38,7 +41,7 @@ ds_deque_t *ds_deque()
 {
     ds_deque_t *deque = ecalloc(1, sizeof(ds_deque_t));
 
-    deque->buffer   = ALLOC_ZVAL_BUFFER(DS_DEQUE_MIN_CAPACITY);
+    deque->buffer   = ds_allocate_zval_buffer(DS_DEQUE_MIN_CAPACITY);
     deque->capacity = DS_DEQUE_MIN_CAPACITY;
     deque->head     = 0;
     deque->tail     = 0;
@@ -65,14 +68,14 @@ static ds_deque_t *ds_deque_from_buffer_ex(
 
 ds_deque_t *ds_deque_from_buffer(zval *buffer, zend_long size)
 {
-    zend_long capacity = ds_deque_required_capacity(size);
+    zend_long capacity = ds_deque_get_capacity_for_size(size);
     return ds_deque_from_buffer_ex(buffer, size, capacity);
 }
 
 ds_deque_t *ds_deque_clone(ds_deque_t *deque)
 {
     zval *source;
-    zval *buffer = ALLOC_ZVAL_BUFFER(deque->capacity);
+    zval *buffer = ds_allocate_zval_buffer(deque->capacity);
     zval *target = buffer;
 
     DS_DEQUE_FOREACH(deque, source) {
@@ -85,7 +88,7 @@ ds_deque_t *ds_deque_clone(ds_deque_t *deque)
 }
 
 
-static inline bool ds_deque_valid_index(ds_deque_t *deque, zend_long index)
+static inline bool ds_deque_valid_position(ds_deque_t *deque, zend_long index)
 {
     if (index < 0 || index >= deque->size) {
         INDEX_OUT_OF_RANGE(index, deque->size);
@@ -103,7 +106,7 @@ void ds_deque_reset_head(ds_deque_t *deque)
     }
 
     if (deque->head < deque->tail) {
-        _memmove(deque, 0, deque->head, deque->size);
+        ds_deque_memmove(deque, 0, deque->head, deque->size);
 
     } else {
         zend_long h = deque->head;
@@ -113,13 +116,13 @@ void ds_deque_reset_head(ds_deque_t *deque)
         // Check if there's enough room to push the left partition forward and
         // the wrapped values (right partition) to the front.
         if (r < (h - t)) {
-            _memmove(deque, r, 0, t);
-            _memmove(deque, 0, h, r);
+            ds_deque_memmove(deque, r, 0, t);
+            ds_deque_memmove(deque, 0, h, r);
 
         } else {
             // We don't have enough temporary space to work with, so create
             // a new buffer, copy to it, then replace the current buffer.
-            zval *buffer = ALLOC_ZVAL_BUFFER(deque->capacity);
+            zval *buffer = ds_allocate_zval_buffer(deque->capacity);
 
             memcpy(&buffer[0], &deque->buffer[h], r * sizeof(zval));
             memcpy(&buffer[r], &deque->buffer[0], t * sizeof(zval));
@@ -135,8 +138,8 @@ void ds_deque_reset_head(ds_deque_t *deque)
 static void ds_deque_reallocate(ds_deque_t *deque, zend_long capacity)
 {
     ds_deque_reset_head(deque);
-    REALLOC_ZVAL_BUFFER(deque->buffer, capacity);
 
+    deque->buffer   = ds_reallocate_zval_buffer(deque->buffer, capacity, deque->size);
     deque->capacity = capacity;
     deque->head     = 0;
     deque->tail     = deque->size;
@@ -147,26 +150,24 @@ static inline void ds_deque_double_capacity(ds_deque_t *deque)
     ds_deque_reallocate(deque, deque->capacity * 2);
 }
 
-static inline void ds_deque_ensure_capacity(ds_deque_t *deque, zend_long size)
+static inline void ds_deque_allocate_capacity_for_size(ds_deque_t *deque, zend_long size)
 {
     if (size >= deque->capacity) {
-        ds_deque_reallocate(deque, ds_deque_required_capacity(size));
+        ds_deque_reallocate(deque, ds_deque_get_capacity_for_size(size));
     }
 }
 
 void ds_deque_allocate(ds_deque_t *deque, zend_long capacity)
 {
     // -1 because an extra slot will be allocated for the tail.
-    ds_deque_ensure_capacity(deque, capacity - 1);
+    ds_deque_allocate_capacity_for_size(deque, capacity - 1);
 }
 
 static inline void ds_deque_auto_truncate(ds_deque_t *deque)
 {
-    // Automatically truncate if the size of the deque is less than a quarter.
-    if (deque->size < deque->capacity / 4) {
-
-        // Truncate to half the capacity, but only if greater than the minimum.
-        if ((deque->capacity / 2) > DS_DEQUE_MIN_CAPACITY) {
+    // Automatically truncate if the size of the deque drops to a quarter.
+    if (deque->size <= deque->capacity / 4) {
+        if (deque->capacity / 2 >= DS_DEQUE_MIN_CAPACITY) {
             ds_deque_reallocate(deque, deque->capacity / 2);
         }
     }
@@ -174,27 +175,18 @@ static inline void ds_deque_auto_truncate(ds_deque_t *deque)
 
 static void ds_deque_clear_buffer(ds_deque_t *deque)
 {
-    zval *value;
+    ds_deque_reset_head(deque);
+    ds_deque_reallocate(deque, DS_DEQUE_MIN_CAPACITY);
 
-    DS_DEQUE_FOREACH(deque, value) {
-        DTOR_AND_UNDEF(value);
-    }
-    DS_DEQUE_FOREACH_END();
-
-    deque->head = 0;
-    deque->tail = 0;
-    deque->size = 0;
+    deque->head   = 0;
+    deque->tail   = 0;
+    deque->size   = 0;
 }
 
 void ds_deque_clear(ds_deque_t *deque)
 {
     if (deque->size > 0) {
         ds_deque_clear_buffer(deque);
-
-        if (deque->capacity > DS_DEQUE_MIN_CAPACITY) {
-            REALLOC_ZVAL_BUFFER(deque->buffer, DS_DEQUE_MIN_CAPACITY);
-            deque->capacity = DS_DEQUE_MIN_CAPACITY;
-        }
     }
 }
 
@@ -223,7 +215,7 @@ static inline zval *ds_deque_lookup(ds_deque_t *deque, zend_long index)
 
 zval *ds_deque_get(ds_deque_t *deque, zend_long index)
 {
-    if ( ! ds_deque_valid_index(deque, index)) {
+    if ( ! ds_deque_valid_position(deque, index)) {
         return NULL;
     }
 
@@ -232,7 +224,7 @@ zval *ds_deque_get(ds_deque_t *deque, zend_long index)
 
 void ds_deque_set(ds_deque_t *deque, zend_long index, zval *value)
 {
-    if (ds_deque_valid_index(deque, index)) {
+    if (ds_deque_valid_position(deque, index)) {
         zval *ptr = ds_deque_lookup(deque, index);
         zval_ptr_dtor(ptr);
         ZVAL_COPY(ptr, value);
@@ -266,7 +258,7 @@ void ds_deque_reverse(ds_deque_t *deque)
 ds_deque_t *ds_deque_reversed(ds_deque_t *deque)
 {
     zval *src;
-    zval *buf = ALLOC_ZVAL_BUFFER(deque->capacity);
+    zval *buf = ds_allocate_zval_buffer(deque->capacity);
     zval *dst = &buf[deque->size - 1];
 
     DS_DEQUE_FOREACH(deque, src) {
@@ -319,7 +311,7 @@ void ds_deque_pop_throw(ds_deque_t *deque, zval *return_value)
 
 void ds_deque_remove(ds_deque_t *deque, zend_long index, zval *return_value)
 {
-    if ( ! ds_deque_valid_index(deque, index)) {
+    if ( ! ds_deque_valid_position(deque, index)) {
         return;
     }
 
@@ -338,12 +330,12 @@ void ds_deque_remove(ds_deque_t *deque, zend_long index, zval *return_value)
     // Translate the positional index to a buffer index.
     index = ds_deque_lookup_index(deque, index);
 
-    // Copy the value into the return value, then destruct.
+    // Copy the value into the return value, then clear it.
     SET_AS_RETURN_AND_UNDEF(&deque->buffer[index]);
 
     if (index < deque->tail) {
         // Shift all values between the index and the tail.
-        _memmove(deque, index, index + 1, deque->tail - index);
+        ds_deque_memmove(deque, index, index + 1, deque->tail - index);
         deque->tail--;
 
     } else {
@@ -351,7 +343,7 @@ void ds_deque_remove(ds_deque_t *deque, zend_long index, zval *return_value)
         // is valid, so it must be after the head which has wrapped around.
 
         // Unshift all values between the head and the index.
-        _memmove(deque, deque->head + 1, deque->head, index - deque->head);
+        ds_deque_memmove(deque, deque->head + 1, deque->head, index - deque->head);
         deque->head++;
     }
 
@@ -361,7 +353,7 @@ void ds_deque_remove(ds_deque_t *deque, zend_long index, zval *return_value)
 
 void ds_deque_unshift_va(ds_deque_t *deque, VA_PARAMS)
 {
-    ds_deque_ensure_capacity(deque, deque->size + argc);
+    ds_deque_allocate_capacity_for_size(deque, deque->size + argc);
     deque->size += argc;
 
     while (argc--) {
@@ -375,14 +367,16 @@ void ds_deque_push(ds_deque_t *deque, zval *value)
     ZVAL_COPY(&deque->buffer[deque->tail], value);
     ds_deque_increment_tail(deque);
 
-    if ((++deque->size) == deque->capacity) {
+    deque->size++;
+
+    if (deque->size == deque->capacity) {
         ds_deque_double_capacity(deque);
     }
 }
 
 void ds_deque_push_va(ds_deque_t *deque, VA_PARAMS)
 {
-    ds_deque_ensure_capacity(deque, deque->size + argc);
+    ds_deque_allocate_capacity_for_size(deque, deque->size + argc);
 
     while (argc--) {
         ZVAL_COPY(&deque->buffer[deque->tail], argv++);
@@ -391,18 +385,35 @@ void ds_deque_push_va(ds_deque_t *deque, VA_PARAMS)
     }
 }
 
-static void _ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS)
+void ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS)
 {
     zval *dst;
     zend_long index;
 
+    // Basic push if inserting at the back.
+    if (position == deque->size) {
+        ds_deque_push_va(deque, VA_ARGS);
+        return;
+    }
+
+    // Basic unshift if inserting at the front.
+    if (position == 0) {
+        ds_deque_unshift_va(deque, VA_ARGS);
+        return;
+    }
+
+    // Check that the insert position is not out of range.
+    if ( ! ds_deque_valid_position(deque, position)) {
+        return;
+    }
+
     // No op if no values.
     if (argc <= 0) {
         return;
     }
 
     // Make sure that we have enough room for the new values.
-    ds_deque_ensure_capacity(deque, deque->size + argc);
+    ds_deque_allocate_capacity_for_size(deque, deque->size + argc);
 
     // Translate the positional index to a buffer index.
     index = ds_deque_lookup_index(deque, position);
@@ -417,7 +428,7 @@ static void _ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS
 
             // There isn't enough free space to the right of the tail,
             // so move the entire sequence all the way to the left.
-            _memmove(deque, 0, deque->head, deque->size);
+            ds_deque_memmove(deque, 0, deque->head, deque->size);
 
             index -= deque->head;
             deque->head = 0;
@@ -426,14 +437,14 @@ static void _ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS
 
         // Move the subsequence after the insertion point to the right
         // to make room for the new values.
-        _memmove(deque, (index + argc), index, (deque->tail - index));
+        ds_deque_memmove(deque, (index + argc), index, (deque->tail - index));
         deque->tail += argc;
         dst = &deque->buffer[index];
 
     } else {
         // We're inserting between a wrapped around head and the end of the
         // buffer, and it's guaranteed that there's enough free space.
-        _memmove(deque, (deque->head - argc), deque->head, (index - deque->head));
+        ds_deque_memmove(deque, (deque->head - argc), deque->head, (index - deque->head));
         deque->head -= argc;
         dst = &deque->buffer[index - argc];
     }
@@ -446,25 +457,6 @@ static void _ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS
     }
 }
 
-void ds_deque_insert_va(ds_deque_t *deque, zend_long position, VA_PARAMS)
-{
-    // Basic push if inserting at the back.
-    if (position == deque->size) {
-        ds_deque_push_va(deque, VA_ARGS);
-        return;
-    }
-
-    // Basic unshift if inserting at the front.
-    if (position == 0) {
-        ds_deque_unshift_va(deque, VA_ARGS);
-        return;
-    }
-
-    if (ds_deque_valid_index(deque, position)) {
-        _ds_deque_insert_va(deque, position, VA_ARGS);
-    }
-}
-
 static zend_long ds_deque_find_index(ds_deque_t *deque, zval *value)
 {
     zend_long head = deque->head;
@@ -692,7 +684,7 @@ ds_deque_t *ds_deque_map(ds_deque_t *deque, FCI_PARAMS)
 {
     zval retval;
     zval *value;
-    zval *buffer = ALLOC_ZVAL_BUFFER(deque->capacity);
+    zval *buffer = ds_allocate_zval_buffer(deque->capacity);
     zval *target = buffer;
 
     DS_DEQUE_FOREACH(deque, value) {
@@ -728,7 +720,7 @@ ds_deque_t *ds_deque_filter_callback(ds_deque_t *deque, FCI_PARAMS)
     } else {
         zval retval;
         zval *value;
-        zval *buffer = ALLOC_ZVAL_BUFFER(deque->capacity);
+        zval *buffer = ds_allocate_zval_buffer(deque->capacity);
         zval *target = buffer;
 
         DS_DEQUE_FOREACH(deque, value) {
@@ -768,7 +760,7 @@ ds_deque_t *ds_deque_filter(ds_deque_t *deque)
         return ds_deque();
 
     } else {
-        zval *buf = ALLOC_ZVAL_BUFFER(deque->capacity);
+        zval *buf = ds_allocate_zval_buffer(deque->capacity);
         zval *dst = buf;
         zval *src = NULL;
 
@@ -823,9 +815,9 @@ ds_deque_t *ds_deque_slice(ds_deque_t *deque, zend_long index, zend_long length)
         return ds_deque();
 
     } else {
-        zend_long capacity = ds_deque_required_capacity(length);
+        zend_long capacity = ds_deque_get_capacity_for_size(length);
 
-        zval *buffer = ALLOC_ZVAL_BUFFER(capacity);
+        zval *buffer = ds_allocate_zval_buffer(capacity);
 
         zval *src = deque->buffer;   // Read from the source deque's buffer
         zval *dst = buffer;          // Pointer to allocated destination buffer
