@@ -78,7 +78,8 @@ static void ds_htable_rehash(ds_htable_t *table)
                 bucket++;
             } while (++index < table->next);
 
-        } else { // There are deleted buckets
+        } else {
+            // There are deleted buckets
             do {
                 if (DS_HTABLE_BUCKET_DELETED(bucket)) {
                     uint32_t j = index;
@@ -128,14 +129,14 @@ static inline void ds_htable_auto_truncate(ds_htable_t *table)
 {
     const uint32_t capacity = table->capacity;
 
-    if (table->size <= capacity / 4 && capacity > DS_HTABLE_MIN_CAPACITY) {
+    if (table->size <= (capacity / 4) && (capacity / 2) >= DS_HTABLE_MIN_CAPACITY) {
         ds_htable_pack(table);
         ds_htable_realloc(table, capacity / 2);
         ds_htable_rehash(table);
     }
 }
 
-ds_htable_t *ds_htable_ex(uint32_t capacity)
+static ds_htable_t *ds_htable_with_capacity(uint32_t capacity)
 {
     ds_htable_t *table = ecalloc(1, sizeof(ds_htable_t));
 
@@ -152,7 +153,7 @@ ds_htable_t *ds_htable_ex(uint32_t capacity)
 
 ds_htable_t *ds_htable()
 {
-    return ds_htable_ex(DS_HTABLE_MIN_CAPACITY);
+    return ds_htable_with_capacity(DS_HTABLE_MIN_CAPACITY);
 }
 
 static void ds_htable_copy(ds_htable_t *_src, ds_htable_t *_dst)
@@ -188,29 +189,28 @@ ds_htable_t *ds_htable_clone(ds_htable_t *src)
 }
 
 static inline bool implements_hashable(zval *key) {
-    return instanceof_function(Z_OBJCE_P(key), hashable_ce);
+    return Z_TYPE_P(key) == IS_OBJECT && instanceof_function(Z_OBJCE_P(key), hashable_ce);
 }
 
-static inline bool user_hashable_equals(zval *a, zval *b)
-{
+ static inline bool user_hashable_equals(zval *a, zval *b)
+ {
     if (Z_OBJCE_P(a) != Z_OBJCE_P(b)) {
         return false;
 
     } else {
-
         zval equals;
         zend_call_method_with_1_params(a, Z_OBJCE_P(a), NULL, "equals", &equals, b);
         return Z_TYPE(equals) == IS_TRUE;
-    }
-}
+     }
+ }
 
-static inline bool key_is_identical(zval *a, zval *b)
+static inline bool key_is_identical(zval *key, zval *other)
 {
-    if (Z_TYPE_P(a) == IS_OBJECT && implements_hashable(a)) {
-        return Z_TYPE_P(b) == IS_OBJECT && user_hashable_equals(a, b);
+    if (Z_TYPE_P(key) == IS_OBJECT && implements_hashable(key)) {
+        return Z_TYPE_P(other) == IS_OBJECT && user_hashable_equals(key, other);
     }
 
-    return zend_is_identical(a, b);
+    return zend_is_identical(key, other);
 }
 
 static inline bool ds_htable_bucket_key_match(ds_htable_bucket_t *bucket, zval *key)
@@ -228,14 +228,14 @@ static inline uint32_t get_string_zval_hash(zval *value)
     return get_string_hash(Z_STR_P(value));
 }
 
-static uint32_t get_array_hash(zval *arr)
+static uint32_t get_array_hash(zval *array)
 {
-    uint32_t                 hash;
+    uint32_t                   hash;
     php_serialize_data_t       var_hash;
     smart_str                  buffer = {0};
 
     PHP_VAR_SERIALIZE_INIT(var_hash);
-    php_var_serialize(&buffer, arr, &var_hash);
+    php_var_serialize(&buffer, array, &var_hash);
     PHP_VAR_SERIALIZE_DESTROY(var_hash);
 
     smart_str_0(&buffer);
@@ -252,9 +252,10 @@ static uint32_t get_array_hash(zval *arr)
 
 static inline uint32_t get_spl_object_hash(zval *obj)
 {
-    zend_string *s = php_spl_object_hash(obj);
-    uint32_t hash = get_string_hash(s);
-    zend_string_free(s);
+    zend_string *str = php_spl_object_hash(obj);
+    uint32_t hash = get_string_hash(str);
+    zend_string_free(str);
+
     return hash;
 }
 
@@ -266,12 +267,10 @@ static inline uint32_t get_resource_hash(zval *resource)
 static uint32_t get_object_hash(zval *obj)
 {
     if (implements_hashable(obj)) {
-
         zval hash;
         zend_call_method_with_0_params(obj, Z_OBJCE_P(obj), NULL, "hash", &hash);
 
         switch (Z_TYPE(hash)) {
-
             case IS_LONG:
                 return Z_LVAL(hash);
 
@@ -301,7 +300,6 @@ static uint32_t get_object_hash(zval *obj)
 static uint32_t get_hash(zval *value)
 {
     switch (Z_TYPE_P(value)) {
-
         case IS_LONG:
             return Z_LVAL_P(value);
 
@@ -373,35 +371,38 @@ ds_htable_bucket_t *ds_htable_lookup_by_position(ds_htable_t *table, uint32_t po
     if (table->size == 0 || position >= table->size) {
         return NULL;
 
+    // A packed table or contiguous section allows us to do a direct lookup.
     } else if (DS_HTABLE_IS_PACKED(table) || position < table->min_deleted) {
         return &table->buckets[position];
 
     } else {
-        ds_htable_bucket_t *bucket;
         uint32_t index;
 
-        // Determine from which end to traverse the buffer
-        if (position > table->size / 2) {
-
-            // Start at the back
-            index = table->size - 1;
-            DS_HTABLE_FOREACH_BUCKET_REVERSED(table, bucket) {
-                if (index-- == position) {
-                    return bucket;
-                }
-            }
-            DS_HTABLE_FOREACH_END();
+        ds_htable_bucket_t *bucket;
+        ds_htable_bucket_t *stop;
 
+        // Skip ahead if to the first deleted bucket if we know we can.
+        if (table->min_deleted <= position) {
+            index = table->min_deleted;
         } else {
-
-            // Start at the front
             index = 0;
-            DS_HTABLE_FOREACH_BUCKET(table, bucket) {
-                if (index++ == position) {
-                    return bucket;
-                }
+        }
+
+        bucket = &table->buckets[index];
+        stop   = &table->buckets[table->next];
+
+        // Scan through the buckets skipping deleted ones until we've counted
+        // enough valid buckets to have reached the one at the given position.
+        for (; bucket < stop; ++bucket) {
+            if (DS_HTABLE_BUCKET_DELETED(bucket)) {
+                continue;
             }
-            DS_HTABLE_FOREACH_END();
+
+            if (position == index) {
+                return bucket;
+            }
+
+            index++;
         }
     }
 
@@ -425,12 +426,14 @@ ds_htable_bucket_t *ds_htable_lookup_by_value(ds_htable_t *table, zval *value)
 bool ds_htable_isset(ds_htable_t *table, zval *key, bool check_empty)
 {
     ds_htable_bucket_t *bucket = ds_htable_lookup_by_key(table, key);
+
     return bucket && ds_zval_isset(&bucket->value, check_empty);
 }
 
 zval *ds_htable_get(ds_htable_t *table, zval *key)
 {
     ds_htable_bucket_t *bucket = ds_htable_lookup_by_key(table, key);
+
     return bucket ? &bucket->value : NULL;
 }
 
@@ -492,6 +495,8 @@ void ds_htable_clear(ds_htable_t *table)
         ds_htable_realloc(table, DS_HTABLE_MIN_CAPACITY);
     }
 
+    ds_htable_reset_lookup(table);
+
     table->min_deleted = table->capacity;
 }
 
@@ -699,7 +704,7 @@ void ds_htable_put(ds_htable_t *table, zval *key, zval *value)
 
 zval *ds_htable_values(ds_htable_t *table)
 {
-    zval *buffer = ALLOC_ZVAL_BUFFER(table->size);
+    zval *buffer = ds_allocate_zval_buffer(table->size);
     zval *target = buffer;
     zval *value;
 
@@ -847,7 +852,7 @@ ds_htable_t *ds_htable_slice(ds_htable_t *table, zend_long index, zend_long leng
         return ds_htable();
 
     } else {
-        ds_htable_t *slice = ds_htable_ex(length);
+        ds_htable_t *slice = ds_htable_with_capacity(length);
 
         /**
          * If the table doesn't have any deleted buckets or if the first deleted
@@ -940,7 +945,7 @@ ds_htable_t *ds_htable_map(ds_htable_t *table, FCI_PARAMS)
     ds_htable_bucket_t *bucket;
     zval retval;
 
-    ds_htable_t *mapped = ds_htable_ex(table->capacity);
+    ds_htable_t *mapped = ds_htable_with_capacity(table->capacity);
 
     DS_HTABLE_FOREACH_BUCKET(table, bucket) {
         fci.param_count = 2;
@@ -966,7 +971,7 @@ ds_htable_t *ds_htable_map(ds_htable_t *table, FCI_PARAMS)
 ds_htable_t *ds_htable_filter(ds_htable_t *table)
 {
     ds_htable_bucket_t *bucket;
-    ds_htable_t *filtered = ds_htable_ex(table->capacity);
+    ds_htable_t *filtered = ds_htable_with_capacity(table->capacity);
 
     DS_HTABLE_FOREACH_BUCKET(table, bucket) {
         if (zend_is_true(&bucket->value)) {
@@ -987,7 +992,7 @@ ds_htable_t *ds_htable_filter_callback(ds_htable_t *table, FCI_PARAMS)
     ds_htable_bucket_t *src;
     zval retval;
 
-    ds_htable_t *filtered = ds_htable_ex(table->capacity);
+    ds_htable_t *filtered = ds_htable_with_capacity(table->capacity);
 
     DS_HTABLE_FOREACH_BUCKET(table, src) {
         fci.param_count = 2;
@@ -1129,7 +1134,7 @@ void ds_htable_reverse(ds_htable_t *table)
 
 ds_htable_t *ds_htable_reversed(ds_htable_t *table)
 {
-    ds_htable_t *reversed = ds_htable_ex(table->capacity);
+    ds_htable_t *reversed = ds_htable_with_capacity(table->capacity);
 
     ds_htable_bucket_t *src = NULL;
     ds_htable_bucket_t *dst = reversed->buckets;
